# Another thing you can do to track down speed problems is to use
# ~olson/MM/Walker/v2/Timer.pm. This package provides a Timer object,
# which contains a number of named timers. To use it, do
# 
# 	$::Timers = new Timer;
# 
# somewhere in the initialization code for each process. Then timers can
# be started and stopped with 
# 
# 	$::Timers->start("timername");
# 	$::Timers->stop("timername");
# 
# The current value of a timer is returned by $::Timers->value("timername").
# 
# Timers can be printed with
# 
# 	$::Timers->print();			# Print all of the timers
# 	$::Timers->print("t1", "t3");		# Just print t1 and t3
# 
# All timers can be reset with
# 
# 	$::Timers->reset();
# 
# Or just some with
# 
# 	$::Timers->reset("tim1", "tim4");

package Timer;
#require HWTime;

$Name = 0;
$StartTime = 1;
$Accum = 2;

sub new
{
    my $self;

    $self = {};
    bless $self;

    return $self;
}

sub value
{
    my $self = shift;
    my ($name) = @_;

    my $timer;

    $timer = $self->_get_timer($name);

    return $$timer[$Accum];
}

sub start
{
    my $self = shift;
    my ($name) = @_;

    my $timer;

    $timer = $self->_get_timer($name);

    if ($$timer[$StartTime] != -1)
    {
	warn "Timer $name not stopped\n";
    }
    else
    {
#	$$timer[$StartTime] = &HWTime::read_hwtime();
        $$timer[$StartTime] = time;
    }
}

sub stop
{
    my $self = shift;
    my ($name) = @_;

    my $timer, $now;

    $timer = $self->_get_timer($name);

    if ($$timer[$StartTime] == -1)
    {
	warn "Timer $name not started\n";
    }
    else
    {
#	$$timer[$Accum] += &HWTime::read_hwtime() - $$timer[$StartTime];
        $$timer[$Accum] += time - $$timer[$StartTime];
	$$timer[$StartTime] = -1;
    }
}

sub stopAndStart
{
    my $self = shift;
    my ($n1, $n2) = @_;

    my $t1, $t2, $now;

    $t1 = $self->_get_timer($n1);
    $t2 = $self->_get_timer($n2);

#    $now = &HWTime::read_hwtime();
    $now = time;

    $$t1[$Accum] += $now - $$t1[$StartTime];
    $$t2[$StartTime] = $now;
}

sub print
{
    my $self = shift;
    my (@which) = @_;
    my $k, $v, $t;

    @which = keys(%$self) unless @which;

    for $k (@which)
    {
	$t = $$self{$k};
	
	print "Timer ", $$t[$Name], ": ", $$t[$Accum], "\n";
    }
}

sub reset
{
    my $self = shift;
    my (@which) = @_;
    my $k, $v, $t;

    @which = keys(%$self) unless @which;

    for $k (@which)
    {
	$t = $$self{$k};

	$$t[$Accum] = 0;
    }
}

sub _get_timer
{
    my $self = shift;
    my ($name) = @_;

    my $timer, $key;

    if (!defined($timer = $$self{$name}))
    {
	$timer = [];
	$$self{$name} = $timer;
	
	$$timer[$Name] = $name;
	$$timer[$StartTime] = -1;
	$$timer[$Accum] = 0;
    }
    return $timer;
}
