#
#

package MDB;
use DB_File;
use Fcntl;
#
# /home/mmlab1/Software/CVSMaster/walker/mdb/MDB.pm,v 1.5 1996/01/18 17:48:03 olson Exp
#
# MDB.pm,v
# Revision 1.5  1996/01/18 17:48:03  olson
# added rcs headers
#
#

#
# Object: MDB
#
#
# Instance variables:
#
#	OBJ_config_file_entries
#
#
# Methods
#
#
#	

@ISA = qw(Object);


#
# Keys
#
$K_status = 'status';
$K_time = 'time';
$K_reference_by = 'reference_by';
$K_reference_to = 'reference_to';
$K_type = 'type';

#
# Values
#
$V_unsearched = 'unsearched';

$first = 1;

sub boot
{
    Nexus::register_handler("new_url", "new_url", 
    			    method_call => 1, unpack_buffer => 1);
    Nexus::register_handler("new_page", "new_page",
			    method_call => 1, unpack_buffer => 1);
    Nexus::register_handler("add_type", "add_type",
                            method_call => 1, unpack_buffer => 1);
    Nexus::register_handler("synch", "synch",
			    method_call => 1, unpack_buffer => 1);
    Nexus::register_handler("index_page", "index_page",
			    method_call => 1, unpack_buffer => 1);
}

sub new
{
    my $self = {};
    bless $self;
    $$self{'OBJ_config_file_entries'} = {};

    $| = 1;

    $self->generate_accessors('hash',
			      'hash_ref',
			      'location',
			      'hdb');

    return $self;
}

#
# Start up the object.
#
sub start
{
    my($self) = @_;

    print "MDB start\n";

    print "start self=$self loc=", $self->location, "\n";
    if ($self->location eq '')
    {
	print "MDB: no location!\n";
	return;
    }
    if (! -d $self->location)
    {
	mkdir($self->location, 0775);
    }

    $db_loc = $self->location . '/hash';

    $flags = 0;
    if (! -f $db_loc)
    {
	$flags |= O_CREAT;
    }
    $flags |= O_RDWR;

    $self->set_hash({});
    my $hr;
    $hr = tie(%{$self->{'_hash'}},
	      DB_File, $db_loc, $flags, 0664,
	      $DB_File::DB_HASH);
    $self->set_hash_ref($hr);
    print "mdb got hashref $hr\n";
    warn "mdb tie failed: $!\n" unless $hr;
}

sub fetch
{
    my($self, $url, $key) = @_;
    my($rc);
    print "Fetching $url $key\n" if $debug;
    $rc = ${$self->hash}{$url, $key};
    print "returns $rc\n" if $debug;
    return $rc;
}

sub store
{
    my($self, $url, $key, $val) = @_;

    print "Storing $url, $key => $val\n" if $debug;
    ${$self->hash}{$url, $key} = $val;
}

sub new_url
{
    my($self, $url, $parent) = @_;
    my($status, $time, $reference_by, $reference_to);

    print "MDB newurl $url\n" if $debug;
    $status = $self->fetch($url, $K_status);
    if (!$status)
    {
	$reference_by = $self->fetch($url, $K_reference_by);
	if ($reference_by)
	{
	    $reference_by .= $; . $parent;
	}
	else
	{
            $reference_by = $parent;
        }
	$self->store($url, $K_reference_by, $reference_by);

	$reference_to = $self->fetch($parent, $K_reference_to);
        if ($reference_to)
	{
	    $reference_to .= $; . $url;
        }
	else
	{
	    $reference_to = $url;
        }
        $self->store($parent, $K_reference_to, $reference_to);

        $self->store($url, $K_time, time);

	print "MDB Add url $url\n" if $debug;
	$self->add_url($url);
    }
    else
    {
	print "URL $url already present\n" if $debug;
    }
}

sub add_url
{
    my($self, $url) = @_;

    $self->store($url, $K_status, $V_unsearched);
}

sub add_type
{
    my($self, $url, $type) = @_;

    $self->store($url, $K_type, $type);
}

sub index_page
{
    my($self, $url_loc) = @_;
    my($rc, $exitcode, $signal);
print "*** We are using glimpseindex\n";
    if ($first)
    {
	$rc = system("$::glimpse_dir/glimpseindex -H $::index_dir $url_loc");
	$first = 0;
    }
    else
    {
        $rc = system("$::glimpse_dir/glimpseindex -a -H $::index_dir $url_loc");
    }	 
    if ($rc != 0)
    {
        my($exitcode, $signal);
        $exitcode = $rc >> 8;
        $signal = $rc & 0xff;
        print "command died with exit code $exitcode signal $signal\n";
    }
}

#
# Enter all the info from this page into the db
#
sub new_page
{
    my($self, $url, $text, $wordlist, $n_headers, $n_links, @rest) = @_;
    my(%headers, @links);
    my($k, $v, $l, $hdb);

    %headers = @rest[0..$n_headers - 1];
    @links = @rest[$n_headers..$#rest];
    $time = time;

    print "New page! url=$url n_headers=$n_headers n_links=$n_links\n" if $debug;
#    print "Headers:\n";
#    while (($k, $v) = each(%headers))
#    {
#	print "    $k $v\n";
#    }
#    print "Links:\n";
#    for $l (@links)
#    {
#	print "    $l\n";
#    }
#    print "Adding\n";
#
    for $l (@links)
    {
	$self->new_url($l, $url);
	$hdb = $self->find_hdb($l);
	Nexus::rpc_async($hdb, "add_url", $l);
    }
}

#
# Eventually this'll get expanded to find the right hdb...
#
sub find_hdb
{
    my($self) = @_;
    return $self->hdb;
}

sub synch
{
    my($self) = @_;
    print "MDB Synch\n";
    return unless ref($self->hash_ref());
    $self->hash_ref()->sync();
}

sub quit
{
    my($self) = @_;
    print "MDB quit\n";
    $self->synch();
    my $r = untie %{$self->{'_hash'}};
    print "untie returns <$r> <$!>\n";
    Nexus::destroy_current_context();
}

sub DESTROY
{
    my($self) = @_;
    $self->quit();
}

1;
